#include "log.hpp"
#include "compmod.hpp"

using namespace std;

/**
 * A logger pointed at stdout;
 */
logger lout(cout);

/**
 * A logger pointed at stderr.
 */
logger lerr(cerr);

logger::logger(ostream &o): os(o) {}

log_type logger::get_log_type(const string &name)
{
   static string log_names[]      = { "model", "report", "status", "res", "dbg", "err", "ext" };
   static log_type log_values[]   = { L_MODEL, L_REPORT, L_STATUS, L_RES, L_DBG, L_ERR, L_EXT };
   
   for(size_t i = 0; i < sizeof(log_values) / sizeof(log_type); i++)
   {
      if(log_names[i] == name)
	 return log_values[i];
   }
   lerr(L_ERR) << "Bad log name" << endl;
   throw compmod::compmod_error();
}

string logger::get_log_name(const log_type &type)
{
   static string log_names[]      = { "model", "report", "status", "res", "dbg", "err", "ext" };
   static log_type log_values[]   = { L_MODEL, L_REPORT, L_STATUS, L_RES, L_DBG, L_ERR, L_EXT };
   
   for(size_t i = 0; i < sizeof(log_values) / sizeof(log_type); i++)
   {
      if(log_values[i] == type)
	 return log_names[i];
   }

   lerr(L_ERR) << "Bad log type" << endl;
   throw compmod::compmod_error();
}

logger &logger::operator()(log_type type)
{
   if(log_types.find(type) == log_types.end())
      logger_valid = false;
   else
      logger_valid = true;
   return *this;
}

void logger::add_log_type(log_type type)
{
   log_types.insert(type);
}

logger &logger::operator<<(ostream &(*pf)(ostream&))
{
   if(logger_valid)
      os << pf;
   return *this;
}

ostream &operator<<(ostream &os, const logger &l)
{
   os << "Logging:";
   set<log_type>::iterator it = l.log_types.begin();
   while(it != l.log_types.end())
   {
      os << " " << logger::get_log_name(*it);
      ++it;
   }
   os << flush;
   return os;
}
