#ifndef RESMES_H
#define RESMES_H

#include <vector>
#include <iostream>
#include <map>
#include <ctime>
#include <sys/resource.h>

/**
 * A resource measurer. By calling start() and stop(), you get
 * measuring periods for this resource.
 */
class resmes
{
public:
   /**
    * Write a text representation of this resource measurement to
    * an ostream. 
    * \param os the ostream to write to.
    * \param res the measurement to write.
    */
   friend std::ostream &operator<<(std::ostream &os, const resmes &res);

   /**
    * Default constructor.
    */
   resmes();

   /**
    * Sets the description of this resource.
    * \param d the description.
    */
   void set_desc(const std::string &d);

   /**
    * Start a measuring period for this resource. This will start
    * the timer. Recursive calls to start will only increase the
    * recursion depth counter.
    */
   void start();
      
   /**
    * Stop a measuring period for this resource. If this is the
    * outermost call (not after a recursive start call), the time
    * between is measured and added to the list of time periods.
    */
   void stop();

private:
      
   /**
    * The description of this measurement.
    */
   std::string desc;

   /**
    * The time at the start of the current measurement.
    */
   timeval start_time;

   /**
    * The current recursion depth.
    */
   size_t depth;

   /**
    * The time periods measured for this resource.
    */
   std::vector<double> periods;
};

extern std::map<std::string, resmes> resources;

#endif // RESMES_H
